#!/usr/bin/env python3
# -*- coding: utf-8 -*-


import numpy as np
import matplotlib.pyplot as plt
from matplotlib import rc
import json

import sys
from pathlib import PurePath
#Start to import others .py code from others directory
lcode = PurePath(__file__)
datalocation = lcode.parents[2].as_posix() + '/'
savelocation = lcode.parents[2].as_posix() + '/figures/'
if  (lcode.parents[1].as_posix() in sys.path)  is False:
    sys.path.insert(0,  lcode.parents[1].as_posix())

import tools.tools as tool


rc('font',**{'family':'sans-serif','sans-serif':['Helvetica']})
rc('text',usetex=True)
plt.rcParams['text.usetex']=True


def load_regret_json(filename, option='regret_only'):
    """
    Load the JSON file with the specified filename and retrieve the following information: 
    mean regret, standard deviation of the regret, the number of steps, the method name, and the number of arms.
    """
    # check the filename given in input
    print(filename)
    f = open(filename,'rb')
    data = json.load(f)
    f.close()

    # load the data from the json file
    if option == 'regret_only':

        mean_regret = np.array(data['mean_result'])
        std_curves = np.array(data['std_result'])
        method = data['methodname']     
        Nbsteps = int(data['Nbsteps'])
        K= int(data['K'])
        try :
            timescale = data['timescale']
            return (mean_regret, std_curves, Nbsteps, method, K,  timescale)      
        except:
            timescale =  np.arange(0, Nbsteps) + 1
            return (mean_regret, std_curves, Nbsteps, method, K, timescale) 
    
    #load thetas value from the json file if needed
    elif option=='thetas': 
          thetas = float(data['thetas'])
          K = float(data['K'])
          return thetas, K

    else:
        print('Nothing loaded please specify option')
        return -1


if __name__ == '__main__' :
    
    # the list of the files to load please enter the name of the file without the .json extension
    # the data files should be located in the folder Data/
    #example
    filenamelist = ['']
    

    color_list = ['blue','red','orange','black','purple', 'green', 'silver', 'magenta', 'darkblue', 'blueviolet', 'brown', 'cadetblue', 'chartreuse', 'darkolivegreen4']
    
    #print('Data/'  + filenamelist[0] + '.json')
    
    results =  { 'regret':[], 'Nsteps':[], 'std':[], 'color':[], 'num':[], 'method':[], 'K':[], 'timescale':[]}     
   
    #load the data from the json file
    for i, filename in enumerate(filenamelist):
        
        filename = datalocation + 'Data/' + filename + '.json'
        mean_result, std_result, Nbsteps, method, K, timescale = load_regret_json(filename, option='regret_only')
        results['regret'].append(mean_result ) 
        results['std'].append(std_result) 
        results['Nsteps'].append(Nbsteps)
        results['color'].append(color_list[i])
        results['method'].append(method)
        results['num'].append(i)
        results['K'].append(K)
        results['timescale'].append(timescale)

    plt.figure()
  
    #plt.gca().set_xscale("log")
      
    #plot the regret loaded from the json files
    for i in range(len(results['Nsteps'])):
        tt = results['timescale'][i]
        yy = results['regret'][i] 
        stdyy = results['std'][i] 
        
        label = tool.get_label(results['method'][i])
        
        plt.plot(tt, yy, linestyle='-', label=label, color = results['color'][i])
        plt.fill_between(tt, yy - stdyy , yy + stdyy, alpha=0.2, 
                         edgecolor=results['color'][i], facecolor=results['color'][i], linewidth=2, linestyle='solid', antialiased=True)
 
    #plt.xlim((0, 100000))   # set the xlim to left, right
    plt.ylim((0,60)) 
    plt.xscale('log')
    plt.title(r'Regret for Bernoulli\textquoteright s law with uniform prior for ' + str(K)  + ' arms', fontsize=12)
    plt.ylabel(r'$Regret$', fontsize=15)
    plt.xlabel(r'$t$', fontsize=15)
    #plt.gca().set_xscale('log')
    #plt.legend(loc=(0.01,0.5), fontsize=6)
    plt.legend(loc='best', fontsize=8)
    
    #plt.xscale('log')
    plt.savefig(savelocation+'compare_regret' + '.pdf', format='pdf')
    plt.show()
    