#!/usr/bin/env python3
# -*- coding: utf-8 -*-


import sys
from pathlib import PurePath
#Start to import others .py code from others directory
lcode = PurePath(__file__)
datalocation = lcode.parents[2].as_posix() + '/'
if  (lcode.parents[1].as_posix() in sys.path)  is False:
    sys.path.insert(0,  lcode.parents[1].as_posix())


#import tools.tools as tls
import bandit_main_classes.algo_bandit_general as algo_general 
import bandit_main_classes.algo_aim as algo_analytic_entropy
import bandit_main_classes.method_bandit as Method


def generate_str_list_of_methodname():
    """
    generate the list of methodname available for the user
    """
    list_str = ' thompson \n'
    list_str += ' aim \n'
    list_str += ' klucb \n'
    list_str += ' klucb_plus_plus \n'
    list_str += ' thompson_plus \n'
    list_str += ' med \n'
    return list_str


# intiate the method and return the bandit method object
def initiate_algo_with_str( method_str, K,  params=None):
    """
    Initialize the bandit algorithm object based on the method name and the number of arms.
    While the 'params' parameter is currently unused, it may become relevant if one wishes 
    to incorporate hyperparameters into the bandit algorithm within the method object. 
    The method object is passed to the bandit algorithm object during its initialization, 
    allowing for the potential sharing of hyperparameters without the need to modify the entire pipeline.
    """
    if method_str == "aim":
        method  = Method.Method_AIM(method_str)
        algo  = algo_analytic_entropy.Algo_AIM(method, K)

    if method_str == "thompson":
        method  =  Method.Method_Thompson_Sampling(method_str)
        algo = algo_general.Algo_Thompson(method, K)  

    if method_str =="klucb_plus_plus": 
        method  =  Method.Method_KLUCB_Plus_Plus(method_str, params[0])
        algo = algo_general.Algo_KLUCB_Plus_Plus(method, K)


    if method_str == "thompson_plus":
        method  =  Method.Method_Thompson_Sampling_Plus(method_str)
        algo = algo_general.Algo_Thompson_Plus(method, K)

    if method_str == "med":
        method  =  Method.Method_MED(method_str)
        algo = algo_general.Algo_MED(method, K)
    
    if method_str =="klucb":
        method  =  Method.Method_KLUCB(method_str)
        algo = algo_general.Algo_KLUCB(method, K)

    return algo, method

# intiate the method and return the bandit method object
def initiate_algo(method, K):
    if method.name == "aim":
        algo  = algo_analytic_entropy.Algo_AIM(method, K)     
    if method.name == "thompson":
        algo = algo_general.Algo_Thompson(method, K)
    if method.name =="klucb_plus_plus": 
        algo = algo_general.Algo_KLUCB_Plus_Plus(method, K)
    if method.name == "thompson_plus":
        algo = algo_general.Algo_Thompson_Plus(method, K)
    if method.name == "med":
        algo = algo_general.Algo_MED(method, K)
    if method.name == "klucb":
        algo = algo_general.Algo_KLUCB(method, K)    
    return  algo

# intiate method and return the bandit method object
def initiate_method(method_str, K, params=None):
    if method_str  == "aim":
        method  = Method.Method_AIM(method_str)
    if method_str =="thompson": 
        method  =  Method.Method_Thompson_Sampling(method_str)   
    if method_str == "thompson_plus":
        method  =  Method.Method_Thompson_Sampling_Plus(method_str)
    if method_str == "med":
        method  =  Method.Method_MED(method_str)
    if method_str =="klucb_plus_plus": 
        method  =  Method.Method_KLUCB_Plus_Plus(method_str, params[0])
    if method_str =="klucb": 
        method  =  Method.Method_KLUCB(method_str)
    return method

