#!/usr/bin/env python3
# -*- coding: utf-8 -*-

#Specific class of method it allows to create method_id object that encapsulate the name but also the parameters of the method itself to avoid a list of parameters inside 

# general method with the minimal number of parametrs needed to initialize the algo
class Method_Bandit :

    """
    A general class for bandit algorithm methods' initialization.
    The random generator is passed to the method object for sharing with the bandit algorithm.
    Specific method classes should allow the creation of method objects, encapsulating the name and additional parameters required by the bandit algorithm, 
    without cluttering the bandit algorithm object with a long list of parameters.
    """
    
    def __init__(self, method_name):
         self.name = method_name
         self.generator = None

class Method_Thompson_Sampling(Method_Bandit):
    """
    Method_Thompson_Sampling is designed for Thompson Sampling algorithm but does not have any additional parameters to pass to the bandit algorithm.
    """
    def __init__(self, method_name):
        super().__init__(method_name)

class Method_AIM(Method_Bandit):
    """
    Method_AIM is specific to AIM and does not require any additional parameters to be passed to the bandit algorithm.
    """
    def __init__(self, method_name):
        super().__init__(method_name)

class Method_KLUCB_Plus_Plus(Method_Bandit):   
    """
    Method_KLUCB_Plus_Plus is designed for KLUCB++ and requires an additional parameter, horizon, to be passed to the bandit algorithm.
    """
    def __init__(self, method_name, horizon):
        super().__init__(method_name)
        self.horizon=horizon

class Method_KLUCB(Method_Bandit):   
    """
    Method_KLUCB is designed for KLUCB and does not require any additional parameters to be passed to the bandit algorithm.
    """
    def __init__(self, method_name):
        super().__init__(method_name)
        
class Method_Thompson_Sampling_Plus(Method_Bandit):
    """
    Method_Thompson_Sampling_Plus is designed for Thompson Sampling Plus and does not require any additional parameters to be passed to the bandit algorithm.
    """
    def __init__(self, method_name):
        super().__init__(method_name)

class Method_MED(Method_Bandit):
    """
    Method_MED is designed for MED and does not require any additional parameters to be passed to the bandit algorithm.
    """
    def __init__(self, method_name):
        super().__init__(method_name)
